@@ -1,17 +1,20 @@
-from django.db import models
 from aislug.helpers import slugify
+from django.db import models
+from django.db.models import Q
+from stringfield import StringField
 
 
-class AISlugField(models.SlugField):
+class AISlugField(StringField):
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault('max_length', 110)
-        kwargs.setdefault('unique', True)
+        kwargs.setdefault('db_index', True)
         kwargs.setdefault('editable', False)
         self.update = kwargs.pop('update', True)
         self.populate_from = kwargs.pop('populate_from', 'title')
         self.slugify = kwargs.pop('slugify', slugify)
         self.invalid = kwargs.pop('invalid', [])
-        self.queryset = kwargs.pop('queryset', None)
+        self.unique_for = kwargs.pop('unique_for', [])
+        if not self.unique_for:
+            kwargs.setdefault('unique', True)
         super(AISlugField, self).__init__(*args, **kwargs)
 
     def pre_save(self, obj, add):
@@ -26,10 +29,16 @@ class AISlugField(models.SlugField):
                 if callable(value):
                     value = value()
         slug = self.slugify(value)
-        if self.queryset is None:
-            queryset = obj.__class__._default_manager
-        else:
-            queryset = self.queryset
+        queryset = obj.__class__._default_manager
+        # filter out the ones that we should make the slug unique for
+        # Note that the slug attr cannot be part of this list.
+        q = Q()
+        for attr in self.unique_for:
+            if attr != self.attname:
+                q = q & Q(**{attr: getattr(obj, attr)})
+        queryset = queryset.filter(q)
+        # This will give us the invalid slugs in one query instead of checking
+        # in each iteration in the below while loop.
         queryset = queryset.filter(**{'%s__startswith' % self.attname: slug})
         if not add:
             queryset = queryset.exclude(pk=obj.pk)
@@ -52,5 +61,5 @@ class AISlugField(models.SlugField):
     def south_field_triple(self):
         from south.modelsinspector import introspector
         args, kwargs = introspector(self)
-        return ('django.db.models.fields.SlugField', args, kwargs)
+        return ('stringfield.base.StringField', args, kwargs)
 
